@@ -20,13 +20,13 @@
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
-#include <libintl.h>
-#define _(s) gettext(s)
 
 #include "immunix.h"
 #include "parser.h"
 #include "profile.h"
 
+typedef int (*comparison_fn_t)(const void *, const void *);
+
 struct alias_rule {
 	char *from;
 	char *to;
@@ -108,7 +108,7 @@ static char *do_alias(struct alias_rule *alias, const char *target)
 
 static Profile *target_prof;
 static struct cod_entry *target_list;
-static void process_entries(const void *nodep, VISIT value, int __unused level)
+static void process_entries(const void *nodep, VISIT value, int level unused)
 {
 	struct alias_rule **t = (struct alias_rule **) nodep;
 	struct cod_entry *entry, *dup = NULL;
@@ -152,7 +152,7 @@ static void process_entries(const void *nodep, VISIT value, int __unused level)
 	}
 }
 
-static void process_name(const void *nodep, VISIT value, int __unused level)
+static void process_name(const void *nodep, VISIT value, int level unused)
 {
 	struct alias_rule **t = (struct alias_rule **) nodep;
 	Profile *prof = target_prof;
