@@ -18,6 +18,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+const char *profile_mode_table[] = {
+	"",
+	"enforce",
+	"complain",
+	"kill",
+	"unconfined",
+};
+
 bool deref_profileptr_lt::operator()(Profile * const &lhs, Profile * const &rhs) const
 {
   return *lhs < *rhs;
@@ -77,6 +85,7 @@ Profile::~Profile()
 {
 	hat_table.clear();
 	free_cod_entries(entries);
+	free_cond_entry_list(xattrs);
 
 	for (RuleList::iterator i = rule_ents.begin(); i != rule_ents.end(); i++)
 		delete *i;
