@@ -20,6 +20,8 @@ my $usestdin = '';
 my %output_rules;
 my $hat = "__no_hat";
 my %flags;
+my %xattrs;
+my $path = '';
 
 GetOptions(
   'escape|E' => \$escape,
@@ -95,9 +97,14 @@ sub gen_default_rules() {
 
   # give every profile access to change_hat
   gen_file("/proc/*/attr/current:w");
+  gen_file("/proc/*/attr/apparmor/current:w");
 
   # give every profile access to /dev/urandom (propolice, etc.)
   gen_file("/dev/urandom:r");
+
+  # give every profile access to FIPS hmac files in /lib and /usr/lib
+  gen_file("/{usr/,}lib{,32,64}/.lib*.so*.hmac:r");
+  gen_file("/{usr/,}lib/{,**/}.lib*.so*.hmac:r");
 }
 
 sub gen_elf_binary($) {
@@ -132,10 +139,10 @@ sub gen_binary($) {
   my $hashbang = head($bin);
   if ($hashbang && $hashbang =~ /^#!\s*(\S+)/) {
     my $interpreter = $1;
-    gen_file("$interpreter:rix");
+    gen_file(realpath($interpreter) . ":rix");
     gen_elf_binary($interpreter);
   } else {
-    gen_elf_binary($bin)
+    gen_elf_binary(realpath($bin))
   }
 }
 
@@ -361,6 +368,7 @@ sub gen_hat($) {
     $hat = $rules[1];
     # give every profile/hat access to change_hat
     @{$output_rules{$hat}} = ( "  /proc/*/attr/current w,\n",);
+    push(@{$output_rules{$hat}}, "  /proc/*/attr/apparmor/current w,\n");
   }
 }
 
@@ -374,6 +382,28 @@ sub gen_addimage($) {
   }
 }
 
+sub gen_xattr($) {
+  my $rule = shift;
+  my @rules = split (/:/, $rule);
+  if (@rules == 3) {
+    $xattrs{$rules[1]} = $rules[2];
+  } elsif (@rules == 2) {
+    $xattrs{$rules[1]} = "";
+  } else {
+    (!$nowarn) && print STDERR "Warning: invalid xattr description '$rule', ignored\n";
+  }
+}
+
+sub gen_path($) {
+  my $rule = shift;
+  my @rules = split (/:/, $rule);
+  if (@rules != 2) {
+    (!$nowarn) && print STDERR "Warning: invalid path description '$rule', ignored\n";
+  } else {
+    $path = $rules[1];
+  }
+}
+
 sub emit_flags($) {
   my $hat = shift;
 
@@ -429,6 +459,10 @@ sub gen_from_args() {
     } elsif ($rule =~ /^addimage:/) {
       gen_addimage($rule);
       $addimage = 1;
+    } elsif ($rule =~ /^xattr:/) {
+      gen_xattr($rule);
+    } elsif ($rule =~ /^path:/) {
+      gen_path($rule);
     } else {
       gen_file($rule);
     }
@@ -438,9 +472,28 @@ sub gen_from_args() {
 
   print STDOUT "# Profile autogenerated by $__VERSION__\n";
   if (not substr($bin, 0, 1) eq "/") {
-	  print STDOUT "profile "
+         print STDOUT "profile "
   }
   print STDOUT "$bin ";
+  if (not $path eq "") {
+    print STDOUT "$path "
+  }
+  if (%xattrs) {
+    print STDOUT "xattrs=(";
+    my $firstloop = 1;
+    foreach my $xattr (keys %xattrs) {
+      if ($firstloop) {
+        $firstloop = 0;
+      } else {
+        print STDOUT " ";
+      }
+      print STDOUT "$xattr";
+      if (not $xattrs{$xattr} eq "") {
+        print STDOUT "=$xattrs{$xattr}";
+      }
+    }
+    print STDOUT ") ";
+  }
   emit_flags('__no_hat');
   print STDOUT "{\n";
   foreach my $outrule (@{$output_rules{'__no_hat'}}) {
